require_relative 'test_helper'

class ContainerTest < MiniTest::Unit::TestCase
  
  def self.before_suite 
    @@name = 'posty-post'
    @@container = TestHelper.setup_posty(@@name)
  end 

  def self.after_suite
    TestHelper.teardown_posty(@@name)
  end

  def test_a_container_that_exits_has_an_id
    container = Docker.run('ubuntu:14.04',nil,'/bin/bash')
    assert_match /[a-z0-9]+/, container.id
    refute_match /[^a-z0-9]/, container.id
    container.remove!
  end

  def test_a_container_run_with_rm_flag_that_exits_returns_removed
    container = Docker.run('ubuntu:14.04', '--rm', '/bin/bash')
    assert_equal "REMOVED", container.id
  end

  def test_a_container_that_echos_to_stdout_has_an_id
    container = Docker.run('ubuntu:14.04',nil,'echo "hello"')
    assert_match /[a-z0-9]+/, container.id
    refute_match /[^a-z0-9]/, container.id
    container.remove!
  end

  def test_container_has_an_id
    assert_match /[a-z0-9]+/, @@container.id
    refute_match /[^a-z0-9]/, @@container.id
    assert_equal 12, @@container.id.length
  end

  def test_stopped_container_is_stopped
    @@container.stop!
    assert @@container.stopped?
    `docker start #{@@name}`
  end

  def test_container_is_up
    assert @@container.up?
  end

  def test_container_is_not_up_when_stopped
    @@container.stop!
    refute @@container.up?
    `docker start #{@@name}`
  end

  def test_container_is_not_up_when_paused
    @@container.pause!
    refute @@container.up?
    `docker unpause #{@@name}`
  end

  def test_container_is_paused
    @@container.pause!
    assert @@container.paused?
    `docker unpause #{@@name}`
  end

  def test_stopped_container_starts
    @@container.stop!
    @@container.start!
    assert @@container.up?
  end

  def test_running_container_restarts
    @@container.restart!
    assert @@container.up?
  end

  def test_container_unpauses
    @@container.pause!
    @@container.unpause!
    assert @@container.up?
  end

  def test_docker_inspect_returns_inspect_object
    assert_instance_of Docker::Inspect, @@container.inspekt
  end

  def test_container_remove_removes_the_container
    shorty = TestHelper.setup_posty('shorty')
    shorty.remove!
    refute Docker.containers.any? { |container| container == shorty }
  end

  def test_container_can_add_removal_instructions
    shorty = TestHelper.setup_posty('shorty')
    shorty.add_removal_instructions do |container|
      ENV["SHORTY_IP"] = container.ip
    end
    assert shorty.removal_instructions, 
      'Shorty should have removal instructions'
    shorty.remove!
    assert ENV["SHORTY_IP"], 'ENV["SHORTY_IP"] should by truthy'
  end

  def test_removed_container_has_methods_disabled
    shorty = TestHelper.setup_posty('shorty')
    shorty.remove!
    [:stop!, :start!, :restart!, :pause!, :unpause!, :remove!]
      .each do |method|
        assert_raises(NoMethodError) do 
          shorty.send(method)
        end
      end
  end
end
